#include "histogram.h"
#include <iostream>
#include <limits>

Input input_data(std::istream& in, bool prompt) {
    Input data;
    size_t number_count;

    if (prompt) std::cerr << "Enter number count: ";
    in >> number_count;

    data.numbers.resize(number_count);
    if (prompt) std::cerr << "Enter " << number_count << " numbers: ";
    for (size_t i = 0; i < number_count; i++) {
        in >> data.numbers[i];
    }

    if (prompt) std::cerr << "Enter bin count: ";
    in >> data.bin_count;

    return data;
}

void find_minmax(const std::vector<double>& numbers, double& min, double& max) {
    min = std::numeric_limits<double>::max();
    max = std::numeric_limits<double>::lowest();

    for (double x : numbers) {
        if (x < min) min = x;
        if (x > max) max = x;
    }
}

std::vector<size_t> make_histogram(const std::vector<double>& numbers, size_t bin_count) {
    double min, max;
    find_minmax(numbers, min, max);

    if (max == min) {
        return std::vector<size_t>(bin_count, numbers.size());
    }

    double bin_size = (max - min) / bin_count;
    std::vector<size_t> bins(bin_count, 0);

    for (double x : numbers) {
        size_t bin_index = static_cast<size_t>((x - min) / bin_size);
        if (bin_index >= bin_count) bin_index = bin_count - 1;
        bins[bin_index]++;
    }

    return bins;
}

void show_histogram_text(const std::vector<size_t>& bins) {
    const size_t SCREEN_WIDTH = 80;
    const size_t MAX_ASTERISK = SCREEN_WIDTH - 3 - 1;
    size_t max_count = 0;

    for (size_t count : bins) {
        if (count > max_count) max_count = count;
    }

    std::cerr << "Bin counts:" << std::endl;

    for (size_t j = 0; j < bins.size(); j++) {
        size_t height = (max_count > MAX_ASTERISK)
            ? static_cast<size_t>(MAX_ASTERISK * (static_cast<double>(bins[j]) / max_count))
            : bins[j];

        if (bins[j] < 10) {
            std::cout << "  " << bins[j] << "|";
        }
        else if (bins[j] < 100) {
            std::cout << " " << bins[j] << "|";
        }
        else {
            std::cout << bins[j] << "|";
        }

        for (size_t k = 0; k < height; k++) std::cout << "*";
        std::cout << std::endl;
    }
}
