#include "histogram.h"
#include "svg.h"
#include <sstream>
#include <curl/curl.h>
#include <string>

size_t write_data(void* ptr, size_t size, size_t nmemb, std::stringstream* stream) {
    size_t data_size = size * nmemb;
    stream->write(static_cast<const char*>(ptr), data_size);
    return data_size;
}

Input download(const std::string& url) {
    std::stringstream buffer;

    CURL* curl = curl_easy_init();
    if (!curl) {
        std::cerr << "Failed to initialize cURL\n";
        exit(1);
    }

    curl_easy_setopt(curl, CURLOPT_URL, url.c_str());
    curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
    curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);

    CURLcode res = curl_easy_perform(curl);
    if (res != CURLE_OK) {
        std::cerr << "cURL error: " << curl_easy_strerror(res) << "\n";
        exit(1);
    }

    curl_easy_cleanup(curl);
    return input_data(buffer, false);
}
int main(int argc, char* argv[]) {
    curl_global_init(CURL_GLOBAL_ALL);

    Input input;
    if (argc > 1) {
        input = download(argv[1]);
    } else {
        input = input_data(std::cin, true);
    }
    auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
    curl_global_cleanup();
    return 0;
}
